(function($, owner) {
    owner.OutputStream = null;
    owner.writer = null;
    
    owner.pageSize = "58mm";
    
    /**
     * 打印纸一行最大的字节
     */
    var LINE_BYTE_SIZE = 32;
    var LINE_BYTE_SIZE_80 = 47;
    var LINE_BYTE_SIZE_110 = 64;
    var LINE_BYTE_SIZE_241 = 100;

    /**
     * 打印三列时，中间一列的中心线距离打印纸左侧的距离
     */
    var LEFT_LENGTH = 16;
    var LEFT_LENGTH_80 = 24;
    var LEFT_LENGTH_110 = 32;

    /**
     * 打印三列时，中间一列的中心线距离打印纸右侧的距离
     */
    var RIGHT_LENGTH = 16;
    var RIGHT_LENGTH_80 = 24;
    var RIGHT_LENGTH_110 = 32;

    /**
     * 打印三列时，第一列汉字最多显示几个文字
     */
    var LEFT_TEXT_MAX_LENGTH = 5;
    var LEFT_TEXT_MAX_LENGTH_80 = 9;
    var LEFT_TEXT_MAX_LENGTH_110 = 11;

    owner.init = function(BluetoothSocket, pageSize) {
        owner.OutputStream = BluetoothSocket.getOutputStream();
        plus.android.importClass(owner.OutputStream);
        
        var OutputStreamWriter = plus.android.importClass('java.io.OutputStreamWriter');
        owner.writer = new OutputStreamWriter(owner.OutputStream, 'GBK'); 
      	plus.android.importClass(owner.writer);
      	
      	owner.pageSize = pageSize;
    }
    
    owner.getBytesLength = function(val) {
            var len = 0;
            for (var i = 0; i < val.length; i++) {
                 var a = val.charAt(i);
                 if (a.match(/[^\x00-\xff]/ig) != null) 
                {
                    len += 2;
                }
                else
                {
                    len += 1;
                }
            }
            return len;
       };
       
    owner.printTwoData = function(leftText, rightText) {
        var sb = leftText;
        
        var leftTextLength = owner.getBytesLength(leftText);
        var rightTextLength = owner.getBytesLength(rightText);
        
        var lineByteSize = LINE_BYTE_SIZE;
        if(owner.pageSize == '80mm'){
        		lineByteSize = LINE_BYTE_SIZE_80;
        }else if(owner.pageSize == '110mm'){
        		lineByteSize = LINE_BYTE_SIZE_110;
        }else if(owner.pageSize == '241mm'){
			lineByteSize = LINE_BYTE_SIZE_241;
		}

        // 计算两侧文字中间的空格
        var marginBetweenMiddleAndRight = parseInt(lineByteSize) - parseInt(leftTextLength) - parseInt(rightTextLength);

        for (var i = 0; i < marginBetweenMiddleAndRight; i++) {
            sb += (" ");
        }
        sb += (rightText);
        
        owner.printString(sb);
    }
    
    owner.printThreeCol = function(leftText, middleText, rightText) {
    	
        var sb = leftText;
        
        var leftTextLength = owner.getBytesLength(leftText);
        var middleTextLength = owner.getBytesLength(middleText);
        var rightTextLength = owner.getBytesLength(rightText)+2;
        
        var leftLength = LEFT_LENGTH;
        var rightLength = RIGHT_LENGTH;
        if(owner.pageSize == '80mm'){
        		leftLength = LEFT_LENGTH_80;
        		rightLength = RIGHT_LENGTH_80;
        }else if(owner.pageSize == '110mm'){
        		leftLength = LEFT_LENGTH_110;
        		rightLength = RIGHT_LENGTH_110;
        }else if(owner.pageSize == '241mm'){
			leftLength = LEFT_LENGTH_241;
			rightLength = RIGHT_LENGTH_241;
		}
        
        //左侧文字
        if(leftTextLength<leftLength){
	    		for(var j=leftTextLength;j<leftLength;j++){
	    			sb += " ";
	    		}
	    	}else{
	    		sb += "\n                ";
	    	}
        
        //中间文字
        sb += (middleText);

        // 计算右侧文字和中间文字的空格长度
        var marginBetweenMiddleAndRight = parseInt(rightLength) - parseInt(middleTextLength / 2) - parseInt(rightTextLength);
        for (var i = 0; i < marginBetweenMiddleAndRight-1; i++) {
            sb += " ";
        }
        
        sb += rightText;

        owner.printString(sb);
    }
    
    owner.printFourCol = function(leftText, middleText,middleText2, rightText) {
    	
        var sb = leftText;
        
        var leftTextLength = owner.getBytesLength(leftText);
        var middleTextLength = owner.getBytesLength(middleText);
        var middleTextLength2 = owner.getBytesLength(middleText2);
        var rightTextLength = owner.getBytesLength(rightText);
        
        var lineByteSize = LINE_BYTE_SIZE;
        if(owner.pageSize == '80mm'){
        		lineByteSize = LINE_BYTE_SIZE_80;
        }else if(owner.pageSize == '110mm'){
        		lineByteSize = LINE_BYTE_SIZE_110;
        }
        
        var avg = Number(lineByteSize)/4;
        
        //左侧文字
        var flag = false; //标记是否换行
        if(leftTextLength<avg){
	    		for(var j=leftTextLength;j<avg;j++){
	    			sb += " ";
	    		}
	    	}else{
	    		flag = true;
	    		sb += "\n";
	    		for(var j=0;j<avg;j++){
	    			sb += " ";
	    		}
	    	}
        
        //中间文字
        sb += (middleText);

        //计算第三列的打印位置
        var length = owner.getBytesLength(sb);
        if(!flag){
	        if(length < Number(avg)*2){
		        for (var i = 0; i < (Number(avg)*2-length); i++) {
		            sb += " ";
		        }
		    }
	    }else{
	    		var marginLength =  avg - middleTextLength - 3;
	    		for (var i = 0; i < (marginLength); i++) {
		        sb += " ";
		    }
	    }
        sb += middleText2;
        
        length = owner.getBytesLength(sb);
        if(!flag){
	        if(length<Number(avg)*3){
		        for (var i = 0; i < (Number(avg)*3-length); i++) {
		            sb += " ";
		        }
		    }
	    }else{
	    		var marginLength =  avg - middleTextLength2 - 3;
	    		for (var i = 0; i < (marginLength); i++) {
		            sb += " ";
		    }
	    }
		sb += rightText;
		
        owner.printString(sb);
    }
    
    owner.printFiveCol = function(leftText, middleText,middleText2, rightText , rightText2) {
    	
        var sb = leftText;
        
        var leftTextLength = owner.getBytesLength(leftText);
        var middleTextLength = owner.getBytesLength(middleText);
        var middleTextLength2 = owner.getBytesLength(middleText2);
        var rightTextLength = owner.getBytesLength(rightText);
        
        var lineByteSize = LINE_BYTE_SIZE;
        if(owner.pageSize == '80mm'){
        		lineByteSize = LINE_BYTE_SIZE_80;
        }else if(owner.pageSize == '110mm'){
        		lineByteSize = LINE_BYTE_SIZE_110;
        }
        
        var avg = Number(lineByteSize)/4;
        
        //左侧文字
        var flag = false; //标记是否换行
        if(leftTextLength<avg){
	    		for(var j=leftTextLength;j<avg;j++){
	    			sb += " ";
	    		}
	    	}else{
	    		flag = true;
	    		sb += "\n";
	    		for(var j=0;j<avg;j++){
	    			sb += " ";
	    		}
	    	}
        
        //中间文字
        sb += (middleText);

        //计算第三列的打印位置
        var length = owner.getBytesLength(sb);
        if(!flag){
	        if(length < Number(avg)*2-2){
		        for (var i = 0; i < (Number(avg)*2-length-2); i++) {
		            sb += " ";
		        }
		    }
	    }else{
	    		var marginLength =  avg - middleTextLength-2;
	    		for (var i = 0; i < (marginLength); i++) {
		        sb += " ";
		    }
	    }
        sb += middleText2;
        
        length = owner.getBytesLength(sb);
        if(!flag){
	        if(length<(Number(avg)*3-4)){
		        for (var i = 0; i < (Number(avg)*3-length-4); i++) {
		            sb += " ";
		        }
		    }
	    }else{
	    		var marginLength =  avg - middleTextLength2;
	    		for (var i = 0; i < (marginLength); i++) {
		            sb += " ";
		    }
	    }
		sb += rightText;
		
		length = owner.getBytesLength(sb);
        if(!flag){
	        if(length<(Number(avg)*3)){
		        for (var i = 0; i < (Number(avg)*3-length); i++) {
		            sb += " ";
		        }
		    }
	    }else{
	    		var marginLength =  avg - middleTextLength2;
	    		for (var i = 0; i < (marginLength); i++) {
		            sb += " ";
		    }
	    }
		sb += rightText2;
		
        owner.printString(sb);
    }
    
    /**
	 * @param {Object} text
	 * @param {Object} n 字体放大倍数
	 */
	owner.buildCenterText = function(text, n){
		var sb = "";
        
		var textLength = owner.getBytesLength(text);
        var m = (LINE_BYTE_SIZE_241/n - textLength) / 2;
        var n = (LINE_BYTE_SIZE_241/n - textLength) % 2;

		for (var j = 0; j < m; j++) {
			sb += (" ");
		}
		sb += (text);
		
		for (var j = 0; j < m; j++) {
			sb += (" ");
		}
		if (n == 1) {
           sb += (" ");
        }
		
        return sb;
	}
	
	//左右两侧填充空格
	owner.fillBlankSpace = function(text, n){
		var sb = "";
		
		for (var j = 0; j < n; j++) {
			sb += (" ");
		}
		
		sb += (text);
		
		for (var j = 0; j < n; j++) {
			sb += (" ");
		}
		
        return sb;
	}
	
	//右侧填充空格
	owner.fillRightBlankSpace = function(text, n){
		var sb = text;
		for (var j = 0; j < n; j++) {
			sb += (" ");
		}
		
        return sb;
	}
	
	//左侧填充空格
	owner.fillLeftBlankSpace = function(text, n){
		var sb = "";
		for (var j = 0; j < n; j++) {
			sb += (" ");
		}
		
        return sb + text;
	}
	
	owner.setRowNum = function(){
    	var reset = [27, 67, 33];
		owner.OutputStream.write(reset);
    };

    // 设置字体大小
    owner.SetFontSize = function(n) {
        var font = [0x1D, 0X21, n]
        owner.OutputStream.write(font);
    };
    
    //设定汉字4倍角打印模式
    owner.setDoubleFont = function() {
       var font = [0x1C, 0x57, 0x01]
       owner.OutputStream.write(font);
    };
	
	//设定取消汉字4倍角打印模式
    owner.setDoubleCancel = function() {
        var font = [0x1C, 0x57, 0x00]
        owner.OutputStream.write(font);
    };

    // 打印字符串
    owner.printString = function(string) {
        var bytes = plus.android.invoke(string, 'getBytes', 'gbk');
        owner.sendData(0, 20, bytes);
    };

	owner.sendData = function(start, end, bytes) {
		try{
			var byteLength = bytes.length;
			owner.OutputStream.write(bytes.slice(start, byteLength > end ? end : byteLength) );
			
			if(byteLength > end){
				owner.sendData(end, end+20 , bytes);
			}
		} catch(e){
			console.log(JSON.stringify(e));
		}
	}
    
    // 打印字符串
    owner.printMark = function(mark) {
    		var string = '';
    		
    		var lineByteSize = LINE_BYTE_SIZE;
    		if(owner.pageSize == '80mm'){
        		lineByteSize = LINE_BYTE_SIZE_80;
        }else if(owner.pageSize == '110mm'){
        		lineByteSize = LINE_BYTE_SIZE_110;
        }else if(owner.pageSize == '241mm'){
	    		lineByteSize = LINE_BYTE_SIZE_241;
	    }
        
    		for(var i=0;i<lineByteSize;i++){
    			string += mark;
    		}
        var bytes = plus.android.invoke(string, 'getBytes', 'gbk');
        owner.OutputStream.write(bytes);
    };
    
     // 重置打印机
    owner.Reset = function() {
        var reset = [0x1B, 0x40];
        owner.OutputStream.write(reset);
    };

    owner.alignCenter = function(){
    	var reset = [0x1b, 0x61, 0x01];
        owner.OutputStream.write(reset);
    };
    
    owner.alignLeft = function(){
    	var reset = [0x1b, 0x61, 0x00];
        owner.OutputStream.write(reset);
    };
    
    owner.alignRight = function(){
    	var reset = [0x1b, 0x61, 0x02];
        owner.OutputStream.write(reset);
    };

    // 打印下划线
    owner.Underline = function() {
        // 下划线指令 
        var underline = [0x1b, 0x2d, 0x01];
        owner.OutputStream.write(underline);
    };

    // 结束打印
    owner.End = function() {
        owner.OutputStream.flush();
        var end = [0x0C];
        owner.OutputStream.write(end);
    };
	
	owner.feedLine = function (n) {
	    const line = n || 1
	    owner.writer.write(0x1B)
	    owner.writer.write(0x64)
	    owner.writer.write(line) // 行数
	    owner.writer.flush()
	};
	
	owner.printBarcode = function(byteStr) {
	
		var vstr = '{B' + byteStr;
		var bytes = plus.android.invoke(vstr, 'getBytes', 'gbk');
		var length = bytes.length;
		
		//条码高度  
		owner.writer.write(29);  
		owner.writer.write(104);  
		owner.writer.write(81); //介于1到255之间，n默认值为162
		owner.writer.flush();
		
		//条码单元宽度
		owner.writer.write(29);  
		owner.writer.write(119);  
		owner.writer.write(3); //介于2到6之间，默认n=3
		owner.writer.flush();
		
		//条码对应字符的打印位置
		owner.writer.write(29);  
		owner.writer.write(72);  
		owner.writer.write(2);  //2表示条形码的下方
		owner.writer.flush();
		
		//输出条形码
		owner.writer.write(29);
		owner.writer.write(107);
		owner.writer.write(73); // 条码类型   code128  
		owner.writer.write(length); //数据长度  
		owner.writer.write(vstr); //条码内容
		owner.writer.flush();
		
	};
    
    // 打印二维码
    owner.printQrCode = function(string){
	
	    var moduleSize = 8;
	    var bytes = plus.android.invoke(string, 'getBytes', 'gbk');
	    var length = bytes.length;
	
	    // 缓存二维码数据
	    owner.writer.write(0x1D);// init  
	    owner.writer.write("(k");// adjust height of barcode  
	    owner.writer.write(length + 3); // pl  
	    owner.writer.write(0); // ph  
	   	owner.writer.write(49); // cn  
	    owner.writer.write(80); // fn  
	    owner.writer.write(48); //  
	    owner.writer.write(string);
	
		// 二维码纠错等级
	    owner.writer.write(0x1D);
	    owner.writer.write("(k");
	    owner.writer.write(3);
	    owner.writer.write(0);
	    owner.writer.write(49);
	    owner.writer.write(69);
	    owner.writer.write(48);
	
		// 设置二维码块大小
	    owner.writer.write(0x1D);
	    owner.writer.write("(k");
	    owner.writer.write(3);
	    owner.writer.write(0);
	    owner.writer.write(49);
	    owner.writer.write(67);
	    owner.writer.write(moduleSize);
	
		// 打印已缓存的数据二维码
	    owner.writer.write(0x1D);
	    owner.writer.write("(k");
	    owner.writer.write(3); // pl  
	    owner.writer.write(0); // ph  
	    owner.writer.write(49); // cn  
	    owner.writer.write(81); // fn  
	    owner.writer.write(48); // m  
	    owner.writer.flush(); 
    };

    // 打印图片（暂不可用）
    owner.Picture = function() {
        var picture = [0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x1B, 0x40, 0x1B, 0x33, 0x00];
        // var picture = [0x1B, 0x2A];
        owner.OutputStream.write(picture);
    };

    // 切纸（暂不可用）
    owner.CutPage = function() {
        // 发送切纸指令  
        var end = [0x1B, 0x69];
        owner.OutputStream.write(end);
    };

    // 条形码打印(暂不可用)
    owner.PrintBarcode = function(n) {
        var barcode = [0x1D, 0x6B, 65, 5, 11, 12, 3, 6, 23];
        owner.OutputStream.write(barcode);
    };
}(mui, window.PrintUtil = {}));


var CommonUtil =  {};  
CommonUtil.WaitFor = function(condition, callback, timeout, unitTime) {
    // 设置默认等待时间（循环间隔）
    if(!unitTime || isNaN(unitTime)) {
        unitTime = 100;
    }
    // 设置超时（到达超时则返回）
    if(!timeout || isNaN(timeout)) {
        timeout = 100;
    }
    if(condition && condition()) { // 等待条件成立，则执行回调
        callback();
    } else if(timeout - unitTime <= 0) { // 等待超时，则执行回调
        callback();
    } else { // 设置延时等待操作
        setTimeout(function() {
            CommonUtil.WaitFor(condition, callback, timeout - unitTime, unitTime);
        }, unitTime);
    }
};

//==========================================================================================


// 导入的java包
var Context, BluetoothAdapter, BluetoothDevice;
// 蓝牙服务与适配
var BManager, BAdapter, BluetoothSocket, mDevice, receiver;
var mMain, mUUID;
var vlist1 = document.getElementById('list1'); //注册容器用来显示未配对设备
var vlist2 = document.getElementById('list2'); //注册容器用来显示未配对设备

/**
 * 1、初始化蓝牙打印设备（第一步必须要调用）
 */
function initBluetooth(){
 
	// 打开蓝牙
	openAndroidBluetooth();

	// 设置延时，防止蓝牙未完全开启时调用
	CommonUtil.WaitFor(
		function() {
			return BAdapter.isEnabled();
		},
		function() {
			// 获取已连接设备列表
			getConnectedDevices();
		},
		1000);

}

/**
 * 2、重新加蓝牙设备列表（刷新按钮调用的方法）
 */
function refreshBluetooth(){
	//buttonRefresh.disabled = true;
    
    if (mui.os.android) {
        // 扫描
        searchDevices();
    }else if(mui.os.ios){
		//TODO:
		
	}
    
    //buttonRefresh.disabled = false;
}

/**
 *打开蓝牙（Android） 
 */
function openAndroidBluetooth() {
    mMain = plus.android.runtimeMainActivity();
    Context = plus.android.importClass("android.content.Context");
    var BManager = mMain.getSystemService(Context.BLUETOOTH_SERVICE);
    plus.android.importClass(BManager); //引入相关的method函数
    BAdapter = BManager.getAdapter();
    plus.android.importClass(BAdapter); //引入相关的method函数，这样之后才会有isEnabled函数支持
    if (!BAdapter.isEnabled()) {
        BAdapter.enable();
    }
}

/**
 * 获取已配对的蓝牙设备列表
 */
function getConnectedDevices() {
    if (!BAdapter.isEnabled()) {
        mui.toast('请先打开蓝牙');
        return;
    }
	
    
    var lists = BAdapter.getBondedDevices(); //获取配对的设备列表
    plus.android.importClass(lists);
    var iterator = lists.iterator();
    plus.android.importClass(iterator);
    var vlist2 = document.getElementById('list2'); //注册容器用来显示未配对设备
    while (iterator.hasNext()) {
        var d = iterator.next();
        plus.android.importClass(d);
        var li2 = genLi(d);
        vlist2.appendChild(li2);
    }
}

/**
 *扫描蓝牙设备 
 */
function searchDevices() {
    //console.log("开始搜索设备");

    if (!BAdapter.isEnabled()) {
        mui.toast('请先打开蓝牙');
        return;
    }

    /*plus.nativeUI.showWaiting('正在搜索设备，请稍后...', {
        back: 'none' // 可取值"none"表示截获处理返回键，但不做任何响应；"close"表示截获处理返回键并关闭等待框；"transmit"表示不截获返回键，向后传递给Webview窗口继续处理（与未显示等待框的情况一致）。 
    });*/
    
    var vlist1 = document.getElementById('list1');
    vlist1.innerHTML = ''; //清空容器
    var vlist2 = document.getElementById('list2');
    vlist2.innerHTML = ''; //清空容器
    
    getConnectedDevices();
   
    // 开启搜索
    BAdapter.startDiscovery();
    
     // 初始化蓝牙广播接收器 
    initReceiver();
    
    // 初始化广播信息过滤
    initIntentFilter();
}

//初始化蓝牙广播接收器 
function initReceiver() {
    BluetoothDevice = plus.android.importClass("android.bluetooth.BluetoothDevice");
    var bdevice = new BluetoothDevice();
    
    receiver = plus.android.implements('io.dcloud.android.content.BroadcastReceiver', {
        onReceive: function(context, intent) { //实现onReceiver回调函数
            plus.android.importClass(intent); //通过intent实例引入intent类，方便以后的‘.’操作

            if (intent.getAction() == "android.bluetooth.adapter.action.DISCOVERY_FINISHED") {
                mMain.unregisterReceiver(receiver); //取消监听
                //console.log("搜索结束")
                plus.nativeUI.closeWaiting();
            } else {
                BleDevice = intent.getParcelableExtra(BluetoothDevice.EXTRA_DEVICE);
                
                //判断是否配对
                if (BleDevice.getBondState() == bdevice.BOND_NONE) {
                    //console.log("未配对蓝牙设备：" + BleDevice.getName() + '    ' + BleDevice.getAddress());
                    if (!document.getElementById(BleDevice.getAddress())) { //判断防止重复添加
                        var li1 = document.createElement('li'); //注册
                        li1.setAttribute('id', BleDevice.getAddress()); //打印机mac地址
	                    li1.setAttribute("class","mui-table-view-cell dashed");
						li2.setAttribute("style","padding: 0px;");
	                    li1.setAttribute('onclick', 'connect(id)'); //注册click点击列表进行配对

						var li11 = document.createElement('a');
						li11.setAttribute("href","#");
						li11.setAttribute("style","padding: 11px 15px;");
						li11.innerText = BleDevice.getName();
						li1.appendChild(li11);
	                    
	                    var vlist1 = document.getElementById('list1');
                        vlist1.appendChild(li1);
                    } 
                } else {
                    if (!document.getElementById(BleDevice.getAddress())) { //判断防止重复添加
                        //console.log("已配对蓝牙设备：" + BleDevice.getName() + '    ' + BleDevice.getAddress());

						var li2 = document.createElement('li'); //注册
		                li2.setAttribute('id', BleDevice.getAddress()); //打印机mac地址
		                li2.setAttribute("class","mui-table-view-cell dashed");
						li2.setAttribute("style","padding: 0px;");
		                li2.setAttribute('onclick', 'print(id)'); //注册click点击列表进行打印
	
						var li22 = document.createElement('a'); 
						li22.setAttribute("href","#");
						li22.setAttribute("style","padding: 11px 15px;");
						li22.innerText = BleDevice.getName();
						li2.appendChild(li22);
                        
                        var vlist2 = document.getElementById('list2');
                        vlist2.appendChild(li2);
                    }
                }
            }
        }
    });
}

//初始化广播信息过滤
function initIntentFilter() {
    // 设置广播信息过滤  
    var IntentFilter = plus.android.importClass('android.content.IntentFilter');
    var filter = new IntentFilter();
    filter.addAction(BluetoothDevice.ACTION_FOUND);
    filter.addAction(BAdapter.ACTION_DISCOVERY_STARTED);
    filter.addAction(BAdapter.ACTION_DISCOVERY_FINISHED);
    filter.addAction(BAdapter.ACTION_STATE_CHANGED);
    // 注册广播接收器，接收并处理搜索结果  
    mMain.registerReceiver(receiver, filter);
}

/**
 * 蓝牙配对
 * @param {Object} mac_address
 */
function connect(mac_address) {
    // 查找蓝牙设备
    var unConnected = BAdapter.getRemoteDevice(mac_address);
    if (unConnected) {
        try {
            if (BleDevice.createBond()) { //配对命令.createBond()
                //console.log("配对成功");
            }
        } catch (e) {
            //TODO handle the exception
            mui.alert('连接到设备失败');
        }
    }
}

/**
 * 
 */
//mac_address:打印机的mac地址
function print(mac_address, str) {
    if (!mac_address) {
        mui.toast('请选择蓝牙打印机');
        return;
    }
    
    if (mDevice == null) {
        UUID = plus.android.importClass("java.util.UUID");
        mUUID = UUID.fromString("00001101-0000-1000-8000-00805F9B34FB");
        mDevice = BAdapter.getRemoteDevice(mac_address);
        plus.android.importClass(mDevice);
        BluetoothSocket = mDevice.createInsecureRfcommSocketToServiceRecord(mUUID);
    }
    
	
    plus.android.importClass(BluetoothSocket);
	
    if (!BluetoothSocket.isConnected()) {
        //console.log('检测到设备未连接，尝试连接....');
        try {
            BluetoothSocket.connect();
        } catch (e) {
			plus.nativeUI.closeWaiting();
            //TODO handle the exception
            mui.alert('设备连接不上，请检查设备状态！');
            return;
        }
    }

    // 防止蓝牙未完成连接时调用打印
    CommonUtil.WaitFor(
        function() { // 等待条件
            return BluetoothSocket.isConnected();
        },
        function() { // 回调方法
            if (!BluetoothSocket.isConnected()) {
                plus.nativeUI.toast('请连接打印机');
                return;
            }
            
            doPrint(); //该方法需要各功能自己实现
            
            mDevice = null //这里关键
        	BluetoothSocket.close(); //必须关闭蓝牙连接否则意外断开的话打印错误
            
        }, 1000);
}


function genLi(bleDevice) {
    var li = document.createElement('li');
    li.setAttribute('id', bleDevice.getAddress());
    li.setAttribute("class","mui-table-view-cell dashed");
	li.setAttribute("style","padding: 0px;");
	li.setAttribute('onclick', 'print(id)'); //注册click点击列表进行打印
	
	var li22 = document.createElement('a');
	li22.setAttribute("href","#");
	li22.setAttribute("style","padding: 11px 15px;");
	li22.innerText = bleDevice.getName();
	li.appendChild(li22);

    return li;
}